package client

import (
	"github.com/go-kit/kit/transport/grpc"
	"go.uber.org/zap"
)

type config struct {
	auth      *authConfig
	noCache   bool
	noLog     bool
	accessLog bool
	debug     bool

	clientOptions []grpc.ClientOption

	logger *zap.Logger
}

type authConfig struct {
	OAuth2 *authOAuth2Config
	TLS    *authTLSConfig
}

type authOAuth2Config struct {
	tokenURL     string
	clientID     string // параметр из auth0 (клиент с таким id должен быть создан в perxis)
	clientSecret string
	audience     string // параметр из auth0 (название связанного с Application API)
}

type authTLSConfig struct {
	cacert []byte
	cert   []byte
	key    []byte
}

type Option func(c *config)

func AuthOAuth2(tokenUrl, clientID, clientSecret, audience string) Option {
	return func(c *config) {
		if c.auth == nil {
			c.auth = &authConfig{}
		}
		c.auth.OAuth2 = &authOAuth2Config{
			tokenURL:     tokenUrl,
			clientID:     clientID,
			clientSecret: clientSecret,
			audience:     audience,
		}
	}
}

func AuthTLS(cacert, cert, key []byte) Option {
	return func(c *config) {
		if c.auth == nil {
			c.auth = &authConfig{}
		}
		c.auth.TLS = &authTLSConfig{
			cacert: cacert,
			cert:   cert,
			key:    key,
		}
	}
}

func NoCache() Option {
	return func(c *config) {
		c.noCache = true
	}
}

func NoLog() Option {
	return func(c *config) {
		c.noLog = true
	}
}

func GrpcClientOptions(opts ...grpc.ClientOption) Option {
	return func(c *config) {
		c.clientOptions = opts
	}
}

func Logger(logger *zap.Logger) Option {
	return func(c *config) {
		c.logger = logger
	}
}

func AccessLog() Option {
	return func(c *config) {
		c.accessLog = true
	}
}
